package vsp;

import java.util.GregorianCalendar;
import java.util.Random;

/**
 * Random numbers generator
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Generator {
	private static final double ROUND = 1000.0;
	private static final int STEPS = 30;
	private Params params;
	private Random random;
	private Trapezoid trapezoid;
	private Box box;
	private Histogram histogram;
	private double[] nums;
	private double sum;
	private GregorianCalendar start;
	private GregorianCalendar end;
	private int totalNumbers;
	
	/**
	 * Creates generator object
	 * @param params parsed command line arguments
	 */
	public Generator(Params params) {
		this.params = params;
		this.nums = null;
		this.random = new Random();
		this.trapezoid = new Trapezoid(params);
		this.sum = -1.0;
		this.histogram = new Histogram(params, STEPS);
		this.box = new Box(params.getStart(), params.getC(), trapezoid.getHeight());
		this.totalNumbers = 0;
	}
	
	/**
	 * Generate random numbers
	 * @return array with random numbers
	 * @throws Exception
	 */
	public double[] generate() throws Exception {
		nums = new double[params.getCount()];
		sum = 0;
		
		start = new GregorianCalendar();
		
		for(int i = 0; i < params.getCount(); i++) {
			nums[i] = generateNumber();
			sum += nums[i];
			histogram.addValue(nums[i]);
		}
		
		end = new GregorianCalendar();
		
		printStatistics();
		
		return nums;
	}
	
	/**
	 * Get real median from generated numbers
	 * @return
	 * @throws Exception
	 */
	public double getRealMedian() throws Exception {
		if (sum < 0) {
			throw new Exception("Generate numbers first!");
		}
		
		return sum / params.getCount();
	}
	
	/**
	 * Get real deviation from generated numbers
	 * @return
	 * @throws Exception
	 */
	public double getRealDeviation() throws Exception {
		if (nums == null) {
			throw new Exception("Generate numbers first!");
		}

		double squareSum = 0;
		double realMedian = this.getRealMedian();

		for (double i : nums) {
			squareSum += Math.pow(realMedian - i, 2);
		}

		return (( 1.0 / ( params.getCount() - 1) ) * squareSum);
	}
	
	/**
	 * Generate next random number
	 * @return random number
	 */
	private double generateNumber() {
		while (true) {
			totalNumbers++;
			
			double xk = box.getA() + (box.getB() - box.getA()) * random.nextDouble();
			double zk = box.getC() * random.nextDouble();
			
			if (zk < trapezoid.functionValue(xk)) {
				return xk;
			}
		}
	}
	
	/**
	 * Print statistics of generation
	 * @throws Exception
	 */
	private void printStatistics() throws Exception {
		System.out.println("E_teorie=" + round(params.getMedian()));
//		System.out.println("D_teorie=" + round(params.getDeviation()));
		System.out.println("E_vypocet=" + round(this.getRealMedian()));
		System.out.println("D_vypocet=" + round(this.getRealDeviation()));
		System.out.println();
		histogram.printHistogram();
		System.out.println();
		long time = end.getTimeInMillis() - start.getTimeInMillis();
		System.out.println(params.getCount() + " generated in " + time + " ms");
		System.out.println("Speed is " + Math.round(params.getCount() * (1000.0 / time)) + " numbers per second");
		System.out.println("Need to generate " + totalNumbers + " numbers");
		System.out.println("Real computer generator speed: " + Math.round(totalNumbers * (1000.0 / time)) + " numbers per second");
		System.out.println("Note: Statistics may not be accurate for short times and for small set of numbers");
	}
	
	/**
	 * Round number to three digits 
	 * @param num common double number
	 * @return double with max three digits
	 */
	public static double round(double num) {
		return Math.round(num * ROUND) / ROUND;
	}
}
